//Global Variable Declarations
var accId = new URLSearchParams(window.location.search).get('accId'); // Get accId from URL parameters
var caseArray;
var helpArray;
var canTranArray;
var csArray;
var table;
var caseType;
const CLASS_ARRAY = 0;
const HELP_ARRAY = 1;

function getCases(accountId, caseArrayType) {
    $("#loadingTable").show();
    if (caseArrayType == CLASS_ARRAY) {
        retrieveMultiple(`${window.location.origin}/_api/accounts?$select=accountid&$expand=incident_customer_accounts($select=incidentid,title,description,casetypecode,_bb_class_value,_bb_educatorlocation_value,statuscode,createdon;$filter=(casetypecode eq 108700005 and statecode eq 0))&$filter=(accountid eq ${accountId})`)
            .then((data) => {
                caseArray = data.value[0]?.incident_customer_accounts;
                generateCaseList(caseArrayType);
            });
    } else if (caseArrayType == HELP_ARRAY) {
        retrieveMultiple(`${window.location.origin}/_api/accounts(${accountId})/incident_customer_accounts?$select=incidentid,title,description,casetypecode,_bb_class_value,_bb_educatorlocation_value,statuscode,createdon&$filter=casetypecode ne 108700005`)
            .then((data) => {
                caseArray = data.value;
                generateCaseList(caseArrayType);
            });
    }
}

//Applicaton ajax wrapper 
function appCaseAjax(ajaxOptions) {
    return webapi.safeAjax(ajaxOptions)
        .fail(function (response) {
            if (response.responseJSON) {
                displayPowerPagesAlert(response.responseJSON.error.message, 'danger');
            } else {
                displayPowerPagesAlert("Error: Web API is not available... ", 'danger');
            }
        });
}

function generateCaseList(caseArrayType) {
    csArray = caseArray;
    caseType = caseArrayType;

    switch (caseArrayType) {
        //CLass Request List
        case CLASS_ARRAY:
            table = new webAPITable({
                columns: [{
                    name: '_bb_class_value@OData.Community.Display.V1.FormattedValue',
                    label: 'Class',
                }, {
                    name: '_bb_educatorlocation_value@OData.Community.Display.V1.FormattedValue',
                    label: 'Location',
                }, {
                    name: 'statuscode@OData.Community.Display.V1.FormattedValue',
                    label: 'Status',
                }, {
                    name: 'createdon@OData.Community.Display.V1.FormattedValue',
                    label: 'Created',
                }],

                data: [],
                sortHandler: sortCaseTable,
            });

            setCaseTableData(undefined, "ClassSchedule-Request");
            break;
        case HELP_ARRAY:
            //Help Request List
            table = new webAPITable({
                columns: [{
                    name: 'title',
                    label: 'Title',
                }, {
                    name: 'casetypecode@OData.Community.Display.V1.FormattedValue',
                    label: 'Case Type',
                }, {
                    name: 'description',
                    label: 'Description',
                }, {
                    name: 'statuscode@OData.Community.Display.V1.FormattedValue',
                    label: 'Status Code',
                }, {
                    name: 'createdon',
                    label: 'Created',
                }],

                data: [],
                sortHandler: sortCaseTable
            });

            setCaseTableData(undefined, "Help");
            break;
        default:
    }
}

function setCaseTableData(filterdData, parentUrl) {
    if (csArray != null && csArray.length > 0) {
        $("#loadingTable").hide();
        $("#emptymessage").hide();
        $("#dataTable").show();
        let dataArray = csArray;

        if (filterdData !== undefined) {
            dataArray = filterdData;
        }

        table.setData(_.map(dataArray, function (record) {
            try {
                record.description = decodeURIComponent(record.description);
            } catch (e) {
                console.error("Error decoding URI component:", e);
                record.description = record.description;
            }

            if (parentUrl === "Help") {
                record.partialUrl = "Help-ReadOnly";
            }
            else if (parentUrl == "ClassSchedule-Request") {
                record.partialUrl = record.statuscode == 108700001 ? "ClassScheduleRequest-Edit" : "ClassScheduleRequest-Read";
            }
            record.parentUrl = parentUrl;
            record.id = record.incidentid;
            let baseUrl = `${window.location.origin}/${parentUrl}/${record.partialUrl}/?id=${record.incidentid}&accId=${accId}`;
            if (record.statuscode == 108700001 && record.casetypecode == 108700005) {
                record.actions = `
                    <td>
                        <div class="dropdown">
                            <button class="btn btn-default dropdown-toggle" type="button" data-toggle="dropdown" style="min-width: 100px;">
                                <span class="caret"></span>
                            </button>
                            <ul class="dropdown-menu">
                                <li><a href="${baseUrl}" class="edit-link launch-modal"><i class="glyphicon glyphicon-pencil" aria-hidden="true"></i> Edit</a></li>
                                <li><a href="#" class="delete"><i class="glyphicon glyphicon-trash" aria-hidden="true"></i> Delete</a></li>
                            </ul>
                        </div>
                    </td>`;
            } else {
                record.actions = `
                    <td>
                        <a href="${baseUrl}" class="btn btn-default" style="min-width: 100px;">
                            <i class="glyphicon glyphicon-eye-open" aria-hidden="true"></i> View
                        </a>
                    </td>`;
            }
            return record;
        }));
    } else {
        $("#loadingTable").hide();
        $("#dataTable").hide();
        $("#emptymessage").show();
    }

    $('#searchInput').on('input', function () {
        var query = $(this).val();
        filterCaseTable(query);
    });

    $('#printButton').on('click', function () {
        printCaseTable();
    });

    $('.delete').on('click', function (e) {
        e.preventDefault();
        dataObj = getCaseDataObject($(this).closest('tr'));
        dataObj.id = $(this).closest('tr')[0].id
        // Add your delete logic here
        console.log('Delete record with id:', dataObj.id);
        deleteCaseRecord(dataObj);
    });
}

var webAPITable = (function () {
    var trTpl = '<% _.forEach(data, function(data){ %>' +
        '<tr id="<%=data.id%>">' +
        '<% _.forEach(columns, function(col){ %>' +
        '<td style="min-width: 100px;" data-attribute="<%=col.name%>" data-label="<%=col.label%>" data-value="<%=data[col.name]%>">' +
        '<a href="<%= window.location.origin %>/<%=data.parentUrl%>/<%=data.partialUrl%>/?id=<%=data.id%>&accId=<%=accId%>"><%-data[col.name]%></a>' +
        '</td>' +
        '<% }) %>' +
        '<%=data.actions%>' +
        '</tr>' +
        '<% }) %>';
    var tableTpl = '<table class="table table-hover">' +
        '<thead>' +
        '<tr>' +
        '<% _.forEach(columns, function(col){ %>' +
        '<th data-column="<%=col.name%>" class="sortable"><%=col.label%><span class="sort-caret"></span></th>' +
        '<% }) %>' +
        '</tr>' +
        '</thead>' +
        '<tbody>' + trTpl + '</tbody>' +
        '</table>';

    return function (config) {
        var me = this,
            columns = config.columns,
            $table;
        me.render = function (el) {
            $table = $(el).html(_.template(tableTpl)({
                columns: columns,
                data: me.data,
                accId: accId
            })).find('table');
            bindCaseTableEvents($table, {
                columns: columns,
                sortHandler: config.sortHandler
            });
        };
        me.addRecord = function (record) {
            $table.find('tbody tr:first').before(_.template(trTpl)({
                columns: columns,
                data: [record],
                accId: accId
            }));
            bindCaseRowEvents($table.find('tbody tr:first'), config);
        };
        me.updateRecord = function (attributeName, newValue, record) {
            $table.find('tr[data-id="' + record.id + '"] td[data-attribute="' + attributeName + '"]').text(newValue);
        };
        me.removeRecord = function (record) {
            $table.find('tr[data-id="' + record.id + '"]').fadeTo("slow", 0.7, function () {
                $(this).remove();
            });
        };
        me.setData = function (data) {
            me.data = data;
            me.render($('#dataTable'));
        };
    };
})();

function deleteCaseRecord(recordObj) {
    var response = confirm("Are you sure, you want to delete \"" + recordObj['_bb_class_value@OData.Community.Display.V1.FormattedValue'] + "\" ?");
    if (response == true) {
        let alertId = displayPowerPagesAlert(`Deleting ${recordObj['_bb_class_value@OData.Community.Display.V1.FormattedValue']}... `, "warning");
        appCaseAjax({
            type: "DELETE",
            url: "/_api/incidents(" + recordObj.id + ")",
            contentType: "application/json",
            success: function (res) {
                csArray = csArray.filter(item => item.incidentid !== recordObj.id);
                setCaseTableData();
                closePowerPagesAlert(alertId);
                displayPowerPagesAlert(`Class Request for ${recordObj['_bb_class_value@OData.Community.Display.V1.FormattedValue']} has successfully deleted.`, "warning", 10000);
            }
        });
    }
    return false;
}

function sortCaseTable(column) {
    var currentOrder = table.currentSortOrder || {};
    var sortDirection = currentOrder[column] === 'asc' ? 'desc' : 'asc';
    currentOrder[column] = sortDirection;
    table.currentSortOrder = currentOrder;

    // This code snippet sorts the data in the table based on the specified column, handling date values for the "createdon" column.
    var sortedData = _.sortBy(table.data, function (item) {
        if (column === "createdon@OData.Community.Display.V1.FormattedValue") {
            return new Date(item[column]);
        } else {
            return item[column];
        }
    });

    if (sortDirection === 'desc') {
        sortedData.reverse();
    }

    if (caseType == 0) {
        setCaseTableData(sortedData, "ClassSchedule-Request");
    }
    else if (caseType == 1) {
        setCaseTableData(sortedData, "Help");
    }

    $('th.sortable').each(function () {
        var $caret = $(this).find('.sort-caret');
        if ($(this).data('column') === column) {
            $caret.text(sortDirection === 'asc' ? '▲' : '▼');
        } else {
            $caret.text('');
        }
    });
}

function filterCaseTable(query) {
    var filteredData = csArray;
    if (query != null && query !== "") {
        filteredData = _.filter(csArray, function (record) {
            return _.some(record, function (value) {
                return value.toString().toLowerCase().includes(query.toLowerCase());
            });
        });
    }

    if (caseType == 0) {
        setCaseTableData(filteredData, "ClassSchedule-Request");
    }
    else if (caseType == 1) {
        setCaseTableData(filteredData, "Help");
    }
}

function printCaseTable() {
    $('#dataTable a').each(function () {
        var $this = $(this);
        $this.replaceWith($this.text());
    });
    var printContent = $('#dataTable').html();
    var originalContent = $('body').html();
    $('body').html(printContent);
    window.print();
    $('body').html(originalContent);
}

function getCurrentDateTimeCase() {
    var now = new Date();
    return now.toISOString();
}

function getCaseDataObject(rowEl) {
    var $rowEl = $(rowEl),
        attrObj = {
            id: $rowEl.attr('data-id'),
            name: $rowEl.attr('data-name')
        };
    $rowEl.find('td').each(function (i, el) {
        var $el = $(el),
            key = $el.attr('data-attribute');
        if (key) {
            attrObj[key] = $el.attr('data-value');
        }
    });
    return attrObj;
}

function bindCaseRowEvents(tr, config) {
    var $row = $(tr),
        dataObj = getCaseDataObject($row);
    dataObj.id = $row[0].id;
    $.each(config.columns, function (i, col) {
        var $el = $row.find('td[data-attribute="' + col.name + '"]');
        $el.on('click', $.proxy(col.handler, $el, col, dataObj));
    });
}

function bindCaseTableEvents($table, config) {
    $table.find('tbody tr').each(function (i, tr) {
        bindCaseRowEvents(tr, config);
    });
    $table.find('th.sortable').on('click', function () {
        var column = $(this).data('column');
        config.sortHandler(column);
    });
}